/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2023 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.atuscriptrecord.service;

import com.alibaba.fastjson.JSONObject;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.scriptcase.dto.atuscriptrecord.*;
import net.northking.cctp.scriptcase.dto.casedesign.TableHeader;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 脚本输入项数据集关联草稿表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface AtuScriptInputSetApiService
{
  /**
   * 查询脚本输入项数据集关联草稿表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<AtuScriptInputSetDetailDto> pagingQuery(QueryByPage<AtuScriptInputSetQueryDto> queryByPage);

  /**
   * 新增脚本输入项数据集关联草稿表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(AtuScriptInputSetAddDto dto);

  /**
   * 更新脚本输入项数据集关联草稿表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(AtuScriptInputSetUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取脚本输入项数据集关联草稿表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  AtuScriptInputSetDetailDto findByPK(String uuid);

  /**
   * 删除脚本输入项数据集关联草稿表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<AtuScriptInputSetAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<AtuScriptInputSetUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  AtuScriptInputSetResultDto queryAtuScriptInputSets(AtuScriptInputSetQueryDto atuScriptInputSetQueryDto);

  AtuScriptInputSetAddDto saveBatch(AtuScriptInputSetAddDto addDto);

  AtuScriptInputSetAddDto saveSceneInputSet(AtuScriptInputSetAddDto dto);

  AtuScriptInputSetResultDto querySetDataByPath(AtuScriptInputSetAddDto addDto);

  JSONObject importExcel(MultipartFile file, List<TableHeader> tableHeaderList, String envId);

  void exportExcel(HttpServletResponse response, AtuScriptInputSetAddTmpDto tmpDto);
}
