/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.elementLibrary.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.dto.AdvancedQueryOptions;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.scriptcase.dto.elementLibrary.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

/**
 * 元素信息管理Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface AtuElementInfoApiService extends ExcelService
{
  /**
   * 查询元素信息管理列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<AtuElementInfoDetailDto> pagingQuery(QueryByPage<AtuElementInfoQueryDto> queryByPage);

  /**
   * 新增元素信息管理
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(AtuElementInfoAddDto dto, String projectId);

  /**
   * 更新元素信息管理
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(AtuElementInfoUpdateDto dto, String projectId) throws IllegalAccessException;

  /**
   * 获取元素信息管理
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  AtuElementInfoDetailDto findByPK(String uuid);

  /**
   * 删除元素信息管理
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量删除
     *
     * @return 成功的数量
     */
    Integer deleteBatch(AtuElementInfoDelDto dto);

  /**
   * 移动元素
   * @param dto
   * @return
   */
    Integer moveElement(AtuElementInfoMoveDto dto);

  /**
   * 图像识别上传图片
   * @param file
   * @return
   */
    String uploadImage(MultipartFile file);

  /**
   * 图像识别展开图片
   * @param response
   * @param url
   */
  void downloadImage(HttpServletResponse response, String url);

    AdvancedQueryOptions advancedQueryOptions(String projectId);
}
