/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.engineInfo.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.device.db.entity.CdEngineInfo;
import net.northking.cctp.device.dto.engineInfo.*;

import java.util.List;

/**
 * 执行引擎信息表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CdEngineInfoApiService extends ExcelService
{
  /**
   * 查询执行引擎信息表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CdEngineInfoDetailDto> pagingQuery(QueryByPage<CdEngineInfoQueryDto> queryByPage);

  /**
   * 新增执行引擎信息表
   *
   * @param dto         接口数据传输对象
   * @return 数据库主键
   */
  String add(CdEngineInfoAddDto dto);

  /**
   * 更新执行引擎信息表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CdEngineInfoUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取执行引擎信息表
   * @param uuid 数据库主键
   * @return 接口数据传输对象
   */
  CdEngineInfoDetailDto findByPK(String uuid);

  /**
   * 删除执行引擎信息表
   * @param uuid  数据库主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<CdEngineInfoAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<CdEngineInfoUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);


    CdEngineInfoResultDto register(CdEngineInfoRegisterDto dto);

    Integer updateVersion(List<String> ids);

    String restart(List<String> list);

    CdEngineInfo getEngineInfo(String proId);

    String releaseDevice(String engineId);

    List<EngineInfoListDto> queryEngineByType(List<DeviceListDto> dto);
}
