/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.model.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.device.dto.model.*;

import java.util.List;
import java.util.Map;

/**
 * 设备型号管理表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CdDeviceModelApiService extends ExcelService
{
  /**
   * 查询设备型号管理表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CdDeviceModelDetailDto> pagingQuery(QueryByPage<CdDeviceModelQueryDto> queryByPage);

  /**
   * 新增设备型号管理表
   *
   * @param dto         接口数据传输对象
   * @return 数据库主键
   */
  String add(CdDeviceModelAddDto dto);

  /**
   * 更新设备型号管理表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CdDeviceModelUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取设备型号管理表
   * @param uuid 数据库主键
   * @return 接口数据传输对象
   */
  CdDeviceModelDetailDto findByPK(String uuid);

  /**
   * 删除设备型号管理表
   * @param uuid  数据库主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<CdDeviceModelAddDto> batch);


    DeviceSelectDto getDeviceCondition(Map<String, Object> queryMap);
}
