/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;

import net.northking.cctp.common.security.authentication.NKSecurityContext;
import net.northking.cctp.testbug.api.service.OverallViewIndexApiService;
import net.northking.cctp.testbug.dto.OverallViewIndexDetailDto;
import net.northking.cctp.testbug.dto.OverallViewIndexQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

@Api(tags = "跨项目级视图索引信息表")
@RequestMapping("/v1")
@RestController
public class OverallViewIndexQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(OverallViewIndexQueryCtrl.class);

  @Autowired
  private OverallViewIndexApiService apiService;

  /**
   * 分页查询 跨项目级视图索引信息表 列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表：跨项目级视图索引信息表")
  @PostMapping(value = "/paging/query/OverallViewIndexs",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<OverallViewIndexDetailDto>> pagingQuery(
      @RequestBody @Validated QueryByPage<OverallViewIndexQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<OverallViewIndexDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<OverallViewIndexDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }

  @ApiOperation(value = "导出Excel：跨项目级视图索引信息表")
  @PostMapping(value = "/excel/export/OverallViewIndexs",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void exportExcel(
      @RequestBody @Validated QueryByPage<OverallViewIndexQueryDto> queryByPage,
      HttpServletResponse response)
  {
    // 定义导出的Excel在客户端浏览器上提示下载的文件名，不写后缀
    String fileName = "CCTP-跨项目级视图索引信息表";
    apiService.exportExcel(response, queryByPage, fileName);
  }

  @ApiOperation(value = "下载Excel模板：跨项目级视图索引信息表")
  @PostMapping(value = "/excel/template/OverallViewIndexs",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void downloadTemplate(HttpServletResponse response)
  {
    String fileName = "CCTP模板-跨项目级视图索引信息表";
    apiService.downloadTemplate(response, fileName);
  }

  @ApiOperation(value = "导入Excel：跨项目级视图索引信息表")
  @PostMapping(value = "/excel/import/OverallViewIndexs",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
  public ResultWrapper<Integer> importExcel(
      @RequestParam("file") MultipartFile file)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    int count = apiService.importExcel(file);

    return wrapper.success(count);
  }

  /**
   * 查询 跨项目级视图索引信息表
   *
   * @param
   * @return 数据列表
   */
  @ApiOperation(value = "用户id：跨项目级视图索引信息表")
  @PostMapping(value = "/query/OverallViewIndex/{recordType}",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<OverallViewIndexDetailDto> oneQuery(
          @PathVariable(value = "recordType" ) String recordType)
  {
    ResultWrapper<OverallViewIndexDetailDto> wrapper = new ResultWrapper<>();
    OverallViewIndexDetailDto overallViewIndex = apiService.oneQuery(NKSecurityContext.getUserId(),recordType);
    return wrapper.success(overallViewIndex);
  }
}


