/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.testbug.db.entity.BugCorrelationLink;
import net.northking.cctp.testbug.dto.*;

import java.util.List;

/**
 * 缺陷关联表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugCorrelationLinkApiService extends ExcelService
{
  /**
   * 查询缺陷关联表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<BugCorrelationLinkDetailDto> pagingQuery(QueryByPage<BugCorrelationLinkQueryDto> queryByPage);

  /**
   * 新增缺陷关联表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(BugCorrelationLinkAddDto dto);

  /**
   * 更新缺陷关联表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(BugCorrelationLinkUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取缺陷关联表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  BugCorrelationLinkDetailDto findByPK(String uuid);

  /**
   * 删除缺陷关联表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugCorrelationLinkAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugCorrelationLinkUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  /**
   * 根据bugId查询关联信息
   * @param bugId
   * @return
   */
  List<BugCorrelationLinkDetailDto> queryByBugId(String bugId);

  /**
   * 查询关联用例信息
   * @param tenantId
   * @return
   */
    List<BugCorrelationLinkDetailDto> queryByTenantId(String tenantId);

    /**
     * 查询缺陷关联信息
     * @param linkId 关联id
     * @return
     */
    List<BugCorrelationLink> listQuery(String linkId);

  void pubAdd(BugCorrelationLink addDto);

  Integer removeByCode(BugLinkDeleteDto batch);
}
