/*
 * Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
 *
 */
package net.northking.cctp.testbug.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.testbug.dto.BugSearchAddDto;
import net.northking.cctp.testbug.dto.BugSearchDetailDto;
import net.northking.cctp.testbug.dto.BugSearchQueryDto;
import net.northking.cctp.testbug.dto.BugSearchUpdateDto;
import net.northking.cctp.testbug.dto.bug.SearchDetailDto;

import java.util.List;

/**
 * 缺陷条件搜索表Restful逻辑接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugSearchApiService extends ExcelService {
    /**
     * 查询缺陷条件搜索表列表
     *
     * @param queryByPage 查询条件
     * @return 查询结果列表
     */
    Pagination<BugSearchDetailDto> pagingQuery(QueryByPage<BugSearchQueryDto> queryByPage);

    /**
     * 新增缺陷条件搜索表
     *
     * @param dto 接口数据传输对象
     * @return 主键
     */
    String add(BugSearchAddDto dto);

    /**
     * 更新缺陷条件搜索表
     *
     * @param dto 接口数据传输对象
     * @return 修改的记录数量
     */
    Integer updateByPK(BugSearchUpdateDto dto) throws IllegalAccessException;

    /**
     * 获取缺陷条件搜索表
     *
     * @param uuid 主键
     * @return 接口数据传输对象
     */
    BugSearchDetailDto findByPK(String uuid);

    /**
     * 删除缺陷条件搜索表
     *
     * @param uuid 主键
     * @return 删除的记录数量
     */
    Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugSearchAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugSearchUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    List<BugSearchDetailDto> pagingQuery2(BugSearchQueryDto queryByPage);

  String addDto(SearchDetailDto inputDto);

    Integer updateSearch(SearchDetailDto inputDto);
}
